@@ -47,7 +47,7 @@
 #'   default for a call to \code{set.seed}.  When the \code{simEd} package is
 #'   detached, \code{base::set.seed} will revert to the default.
 #'
-#' @return  \code{set.seed} returns \code{NULL}, invisibly, consistent with
+#' @returns  \code{set.seed} returns \code{NULL}, invisibly, consistent with
 #'          \code{base::set.seed}.
 #'
 #' @seealso \code{\link[base:set.seed]{base::set.seed}}
@@ -116,6 +116,217 @@ set.seed <- function(seed, kind = NULL, normal.kind = NULL)
 }
 ################################################################################
 
+################################################################################
+## sample: streams-capable version of base::sample
+##   If stream == NULL, invokes base::sample directly;
+##     o/w, uses vunif to sample across provided x.
+##   Unlike base::sample, we require sum(prob) to be 1.
+################################################################################
+#' Random Samples
+#'
+#' @description   \code{sample} takes a sample of the specified size from the
+#'     elements of \code{x}, either with or without replacement, and with
+#'     capability to use independent streams and antithetic variates in the draws.
+#'
+#' @param x         Either a vector of one or more elements from which to choose,
+#'                    or a positive integer
+#' @param size      A non-negative integer giving the number of items to choose
+#' @param replace   If \code{FALSE} (default), sampling is without replacement;
+#'                    otherwise, sample is with replacement
+#' @param prob      A vector of probability weights for obtaining the elements
+#'                    of the vector being sampled
+#' @param stream    If \code{NULL} (default), directly calls \code{base::sample}
+#'                    and returns its result; otherwise, an integer in 1:100
+#'                    indicates the \code{rstream} stream used to generate the sample
+#' @param antithetic If \code{FALSE} (default), uses \eqn{u} = uniform(0,1)
+#'                    variate(s)generated via \code{rstream::rstream.sample} to
+#'                    generate the sample; otherwise, uses \eqn{1 - u}.
+#'                    (NB: ignored if \code{stream} is \code{NULL}.)
+#'
+#' @details
+#'    If \code{stream} is \code{NULL}, sampling is done by direct call to
+#'    \code{\link[=sample.int]{base::sample}} (refer to its documentation for details).
+#'    In this case, a value of \code{TRUE} for \code{antithetic} is ignored.
+#'
+#'    The remainder of details below presume that \code{stream} has a positive
+#'    integer value, corresponding to use of the \code{\link{vunif}} variate
+#'    generator for generating the random sample.
+#'
+#'    If \code{x} has length 1 and is numeric, sampling takes place from \code{1:x}
+#'    only if \code{x} is a positive integer; otherwise, sampling takes place using
+#'    the single value of \code{x} provided (either a floating-point value or a
+#'    non-positive integer).  Otherwise \code{x} can be a valid R vector, list, or
+#'    data frame from which to sample.
+#'
+#'    The default for \code{size} is the number of items inferred from \code{x},
+#'    so that \code{sample(x, stream = }\eqn{m}\code{)} generates a
+#'    random permutation of the elements of \code{x} (or \code{1:x}) using random
+#'    number stream \eqn{m}.
+#'
+#'    It is allowed to ask for \code{size = 0} samples (and only then is a
+#'    zero-length \code{x} permitted), in which case
+#'    \code{\link[=sample.int]{base::sample}} is invoked to return the correct
+#'    (empty) data type.
+#'
+#'    The optional \code{prob} argument can be used to give a vector of probabilities
+#'    for obtaining the elements of the vector being sampled. Unlike
+#'    \code{\link[=sample.int]{base::sample}}, the weights here must sum to one.
+#'    If \code{replace} is false, these probabilities are applied successively;
+#'    that is the probability of choosing the next item is proportional to the
+#'    weights among the remaining items. The number of nonzero probabilities must
+#'    be at least \code{size} in this case.
+#'
+#' @returns
+#'    If \code{x} is a single positive integer, \code{sample} returns a vector
+#'      drawn from the integers \code{1:x}.
+#'    Otherwise, \code{sample} returns a vector, list, or data frame consistent
+#'      with \code{typeof(x)}.
+#'
+#' @seealso \code{\link[=sample.int]{base::sample}}, \code{\link{vunif}}
+#' @template signature
+#' @keywords distribution
+#' @concept  random sampling
+#'
+#' @examples
+#'  set.seed(8675309)
+#'
+#'  # use base::sample (since stream is NULL) to generate a permutation of 1:5
+#'  sample(5)
+#'
+#'  # use vunif(1, stream = 1) to generate a permutation of 1:5
+#'  sample(5, stream = 1)
+#'
+#'  # generate a (boring) sample of identical values drawn using the single value 867.5309
+#'  sample(867.5309, size = 10, replace = TRUE, stream = 1)
+#'
+#'  # use vunif(1, stream = 1) to generate a size-10 sample drawn from 7:9
+#'  sample(7:9, size = 10, replace = TRUE, stream = 1)
+#'
+#'  # use vunif(1, stream = 1) to generate a size-10 sample drawn from c('x','y','z')
+#'  sample(c('x','y','z'), size = 10, replace = TRUE, stream = 1)
+#'
+#'  # use vunif(1, stream = 1) to generate a size-5 sample drawn from a list
+#'  mylist <- list()
+#'  mylist$a <- 1:5
+#'  mylist$b <- 2:6
+#'  mylist$c <- 3:7
+#'  sample(mylist, size = 5, replace = TRUE, stream = 1)
+#'
+#'  # use vunif(1, stream = 1) to generate a size-5 sample drawn from a data frame
+#'  mydf <- data.frame(a = 1:6, b = c(1:3, 1:3))
+#'  sample(mydf, size = 5, replace = TRUE, stream = 1)
+#'
+#' @export
+################################################################################
+sample <- function(
+  x, size, replace = FALSE, prob = NULL,
+  stream = NULL, antithetic = FALSE
+) {
+   # if stream is NULL, just let base::sample do the work
+   if (is.null(stream)) {
+      if (antithetic == TRUE) {
+         # mod 23 Nov 2023
+         #warning("ignoring antithetic = TRUE since stream = NULL invokes base::sample")
+         warning("ignoring antithetic = TRUE since stream = NULL invokes base::sample",
+                 immediate. = TRUE)
+      }
+      return( base::sample(x, size, replace, prob) )
+   }
+
+   # if size is 0, just let base::sample return the correct empty type...
+   if (!missing(size) && size == 0) return( base::sample(x, size, replace, prob) )
+
+   ############################################################################
+
+   # not calling base::sample, so do some error checking...
+   if (missing(x) || length(x) < 1)
+      stop(paste("'x' must be a positive integer or a vector of",
+                    "one or more elements from which to choose"))
+
+   if (is.numeric(x) && length(x) == 1 && floor(x) == x && x >= 1)
+   {
+      x <- 1:x  # make x a vector of ints from 1:x for replace logic below...
+   }
+
+   if (missing(size)) size <- length(x)
+   if (length(size) != 1 || !is.numeric(size) || size < 0 || floor(size) != size)
+      stop("'size' must be a non-negative integer")
+   if (replace == FALSE && size > length(x))
+      stop("cannot take a sample larger than the population when 'replace = FALSE'")
+
+   if (!is.logical(replace)) stop("'replace' must be a logical value")
+
+   # different requirements than base::sample -- sum(prob) must be 1...
+   if (!is.null(prob)) {
+      if (length(prob) != length(x))
+         stop("incorrect number of probabilities")
+      if (!is.numeric(prob) || sum(prob) != 1 || any(prob < 0))
+         stop("'prob' must be a vector of non-negative real values that sum to 1")
+      if (replace == FALSE && length(which(prob > 0)) < size)
+         stop("too few positive probabilities")
+   }
+
+   if (!(is.null(stream) || is.numeric(stream)))
+      stop("'stream' must be NULL or a positive integer")
+   if (!is.null(stream) &&
+      (floor(stream) != stream || stream <= 0 ||
+                                  stream > simEd_env$simEd_max_streams))
+      stop(paste("'stream' must be a positive integer no greater than",
+         simEd_env$simEd_max_streams))
+
+   if (!is.logical(antithetic)) stop("'antithetic' must be TRUE or FALSE")
+
+   ############################################################################
+
+   if (is.data.frame(x) || is.list(x))
+      # create an empty list of appropriate size; convert @ end if data.frame
+      theSample <- lapply(1:size, function(x) { NULL } )
+   else
+      theSample <- rep(NA, size)  # vector
+
+   if (!is.null(prob))
+      sumProb <- cumsum(prob)
+
+   for (i in 1:size) {
+      u <- vunif(1, 0, 1, stream = stream)
+      if (antithetic == TRUE) u <- 1 - u
+
+      if (!is.null(prob))
+         idx <- which(u <= sumProb)[1]     # find 1st index with cumsum value > u
+      else
+         idx <- 1 + floor(length(x) * u)  # o/w, map u uniformly across range
+
+      if (is.data.frame(x) || is.list(x)) {
+         # add a new entry to the appropriate list entry, and update its name
+         theSample[[i]]      <- x[[idx]]  # use of [[idx]] drops extraneous name
+         names(theSample)[i] <- names(x)[idx]
+      } else {
+         theSample[i] <- x[idx]  # plop the selected item into sample vec
+      }
+
+      if (replace == FALSE) {
+         x <- x[-idx]  # remove the x entry at position idx
+
+         if (!is.null(prob)) {
+            # update the probabilities appropriately by removing the
+            # corresponding probability, and then accounting for its removal...
+            p    <- prob[idx]
+            prob <- prob[-idx] / (1 - p)    # need to scale remaining probs
+            sumProb <- cumsum(prob)
+            sumProb[length(sumProb)] <- 1   # just in case roundoff error...
+         }
+      }
+   }
+
+   # if x is data frame, convert theSample, which handles column naming by
+   # auto-appending .1, .2, etc. as necessary
+   if (is.data.frame(x)) theSample <- as.data.frame(theSample)
+
+   return(theSample)
+}
+################################################################################
+
+
 
 
 ################################################################################
@@ -424,7 +635,10 @@ vexp <- function(n,
       "u"        = u,
       "x"        = qexp(u, rate = rate),
       "quantile" = function(i) qexp(i, rate = rate),
-      "text"     = paste("Exp (", sym$lambda, " = ", round(rate, 3), ")\n", sep = "")
+      # using plotmath for lambda; bquote to use .() to evaluate args;
+      #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+      "text"     = as.expression(bquote("Exp (" ~ lambda ~ "=" ~ .(round(rate, 3)) ~ ")\n"))
+      #"text"     = paste("Exp (", sym$lambda, " = ", round(rate, 3), ")\n", sep = "")
     ))
   }
 
@@ -484,17 +698,31 @@ vfd <- function(n,
           function(d) qf(d, df1, df2)
     else  function(d) qf(d, df1, df2, ncp)
 
+  # using plotmath for nu, delta; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  if (!missing(ncp)) {
+    text_ <- as.expression(bquote(
+                    "F (" ~ nu * "1" ~ "=" ~ .(round(df1, 3)) * ","
+                          ~ nu * "2" ~ "=" ~ .(round(df2, 3)) * ","
+                          ~ delta    ~ "=" ~ .(round(ncp, 3)) ~ ")\n"))
+  } else {
+    text_ <- as.expression(bquote(
+                    "F (" ~ nu * "1" ~ "=" ~ .(round(df1, 3)) * ","
+                          ~ nu * "2" ~ "=" ~ .(round(df2, 3)) ~ ")\n"))
+  }
+
   if (asList) {
     return(list(
       "u"        = u,
       "x"        = qfunc(u),
       "quantile" = function(i) qfunc(i),
-      "text"     = paste("F (",
-                      sym$nu, "1 = ", round(df1, 3), ", ",
-                      sym$nu, "2 = ", round(df2, 3),
-                      (if (!missing(ncp))
-                          paste(",", sym$delta, "=", round(ncp, 3))),
-                      ")\n", sep = "")
+      "text"     = text_
+      #"text"     = paste("F (",
+      #                sym$nu, "1 = ", round(df1, 3), ", ",
+      #                sym$nu, "2 = ", round(df2, 3),
+      #                (if (!missing(ncp))
+      #                    paste(",", sym$delta, "=", round(ncp, 3))),
+      #                ")\n", sep = "")
     ))
   }
 
@@ -550,19 +778,32 @@ vgamma <- function(n,
   # if antithetic requested, transform u
   if (antithetic == TRUE) u <- 1 - u
 
+  # using plotmath for alpha, beta, theta; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  if (missing(rate)) {
+    text_ <- as.expression(bquote(
+                    Gamma ~ "(" ~ "k"   ~ "=" ~ .(round(shape, 3)) * ","
+                                ~ theta ~ "=" ~ .(round(scale, 3)) ~ ")\n"))
+  } else {
+    text_ <- as.expression(bquote(
+                    Gamma ~ "(" ~ alpha ~ "=" ~ .(round(shape, 3)) * ","
+                                ~ beta  ~ "=" ~ .(round(rate,  3)) ~ ")\n"))
+  }
+
   if (asList) {
     return(list(
       "u"        = u,
       "x"        = qgamma(u, shape = shape, scale = scale),
       "quantile" = function(i) qgamma(i, shape = shape, scale = scale),
-      "text"     = paste(sym$Gamma, " (",
-           if (missing(rate))
-             paste("k = ",           round(shape, 3), ", ",
-                   sym$theta, " = ", round(scale, 3), sep = "")
-           else
-             paste(sym$alpha, " = ", round(shape, 3), ", ",
-                   sym$beta,  " = ", round(rate,  3), sep = ""),
-           ")\n", sep = "")
+      "text"     = text_
+      #"text"     = paste(sym$Gamma, " (",
+      #     if (missing(rate))
+      #       paste("k = ",           round(shape, 3), ", ",
+      #             sym$theta, " = ", round(scale, 3), sep = "")
+      #     else
+      #       paste(sym$alpha, " = ", round(shape, 3), ", ",
+      #             sym$beta,  " = ", round(rate,  3), sep = ""),
+      #     ")\n", sep = "")
     ))
   }
 
@@ -617,7 +858,8 @@ vgeom <- function(n,
       "u"        = u,
       "x"        = qgeom(u, prob = prob),
       "quantile" = function(i) qgeom(i, prob = prob),
-      "text"     = paste("Geom (p = ", round(prob, 3), ")\n", sep = "")
+      "text"     = as.expression(bquote("Geom (" ~ "p" ~ "=" ~ .(round(prob, 3)) ~ ")\n"))
+      #"text"     = paste("Geom (p = ", round(prob, 3), ")\n", sep = "")
     ))
   }
 
@@ -628,7 +870,7 @@ vgeom <- function(n,
 
 
 ################################################################################
-#  vgeom - Variate Generation for Log-Normal Distribution
+#  vlnorm - Variate Generation for Log-Normal Distribution
 # ------------------------------------------------------------------------------
 #' @templateVar distro   Log-Normal
 #' @templateVar distrolc log-normal
@@ -674,10 +916,13 @@ vlnorm <- function(n,
       "u"        = u,
       "x"        = qlnorm(u, meanlog = meanlog, sdlog = sdlog),
       "quantile" = function(i) qlnorm(i, meanlog = meanlog, sdlog = sdlog),
-      "text"     = paste("Lognormal (",
-                            sym$mu,    " = ", round(meanlog, 3), ", ",
-                            sym$sigma, " = ", round(sdlog, 3),
-                            ")\n", sep = "")
+      "text"     = as.expression(bquote(
+                        "Lognormal (" ~ mu    ~ "=" ~ .(round(meanlog, 3)) * ","
+                                      ~ sigma ~ "=" ~ .(round(sdlog,   3)) ~ ")\n"))
+      #"text"     = paste("Lognormal (",
+      #                      sym$mu,    " = ", round(meanlog, 3), ", ",
+      #                      sym$sigma, " = ", round(sdlog, 3),
+      #                      ")\n", sep = "")
     ))
   }
 
@@ -734,10 +979,13 @@ vlogis <- function(n,
       "u"        = u,
       "x"        = qlogis(u, location = location, scale = scale),
       "quantile" = function(i) qlogis(i, location = location, scale = scale),
-      "text"     = paste("Logistic (",
-                           sym$mu,    " = ", round(location, 3), ", ",
-                           sym$sigma, " = ", round(scale, 3), ")\n",
-                           sep = "")
+      "text"     = as.expression(bquote(
+                        "Logistic (" ~ mu    ~ "=" ~ .(round(location, 3)) * ","
+                                     ~ sigma ~ "=" ~ .(round(scale,    3)) ~ ")\n"))
+      #"text"     = paste("Logistic (",
+      #                     sym$mu,    " = ", round(location, 3), ", ",
+      #                     sym$sigma, " = ", round(scale, 3), ")\n",
+      #                     sep = "")
     ))
   }
 
@@ -798,16 +1046,29 @@ vnbinom <- function(n,
           function(d) qnbinom(d, size, prob = prob)
     else  function(d) qnbinom(d, size, mu = mu)
 
+  # using plotmath for mu; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  if (missing(mu)) {
+    text_ <- as.expression(bquote(
+                    "NBinomial (" ~ "r" ~ "=" ~ .(round(size, 3)) * ","
+                                  ~ "p" ~ "=" ~ .(round(prob, 3)) ~ ")\n"))
+  } else {
+    text_ <- as.expression(bquote(
+                    "NBinomial (" ~ "r" ~ "=" ~ .(round(size, 3)) * ","
+                                  ~ mu  ~ "=" ~ .(round(mu,   3)) ~ ")\n"))
+  }
+
   if (asList) {
     return(list(
       "u"        = u,
       "x"        = qfunc(u),
       "quantile" = function(i) qfunc(i),
-      "text"     = paste("NBinomial (",
-                     "r = ", round(size, 3), ", ",
-                     (if(is.null(mu))  paste("p =",       round(prob, 3))
-                      else             paste(sym$mu, "=", round(mu, 3))
-                     ), ")\n", sep = "")
+      "text"     = text_
+      #"text"     = paste("NBinomial (",
+      #               "r = ", round(size, 3), ", ",
+      #               (if(is.null(mu))  paste("p =",       round(prob, 3))
+      #                else             paste(sym$mu, "=", round(mu, 3))
+      #               ), ")\n", sep = "")
     ))
   }
 
@@ -866,10 +1127,13 @@ vnorm <- function(n,
       "u"        = u,
       "x"        = qnorm(u, mean = mean, sd = sd),
       "quantile" = function(i) qnorm(i, mean = mean, sd = sd),
-      "text"     = paste("Normal (",
-                     sym$mu,    " = ", round(mean, 3), ", ",
-                     sym$sigma, " = ", round(sd,   3), ")\n",
-                     sep = "")
+      "text"     = as.expression(bquote(
+                        "Normal (" ~ mu    ~ "=" ~ .(round(mean, 3)) * ","
+                                   ~ sigma ~ "=" ~ .(round(sd,   3)) ~ ")\n"))
+      #"text"     = paste("Normal (",
+      #               sym$mu,    " = ", round(mean, 3), ", ",
+      #               sym$sigma, " = ", round(sd,   3), ")\n",
+      #               sep = "")
     ))
   }
 
@@ -926,8 +1190,10 @@ vpois <- function(n,
       "u"        = u,
       "x"        = qpois(u, lambda = lambda),
       "quantile" = function(i) qpois(i, lambda = lambda),
-      "text"     = paste("Poisson (", sym$lambda, " = ", round(lambda, 3),
-                       ")\n", sep = "")
+      "text"     = as.expression(bquote(
+                        "Poisson (" ~ lambda ~ "=" ~ .(round(lambda, 3)) ~ ")\n"))
+      #"text"     = paste("Poisson (", sym$lambda, " = ", round(lambda, 3),
+      #                 ")\n", sep = "")
     ))
   }
 
@@ -985,15 +1251,27 @@ vt <- function(n,
           function(d) qt(d, df)
     else  function(d) qt(d, df, ncp)
 
+  # using plotmath for nu, sigma; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  if (!is.null(ncp)) {
+    text_ <- as.expression(bquote(
+                    "t (" ~ nu    ~ "=" ~ .(round(df,  3)) * ","
+                          ~ sigma ~ "=" ~ .(round(ncp, 3)) ~ ")\n"))
+  } else {
+    text_ <- as.expression(bquote(
+                    "t (" ~ nu    ~ "=" ~ .(round(df,  3)) ~ ")\n"))
+  }
+
   if (asList) {
     return(list(
       "u"        = u,
       "x"        = qfunc(u),
       "quantile" = function(i) qfunc(i),
-      "text"     = paste("t (",
-                     sym$nu, " = ", round(df, 3),
-                     (if(!is.null(ncp)) paste(",", sym$sigma, "=", round(ncp,3))
-                     ), ")\n", sep = "")
+      "text"     = text_
+      #"text"     = paste("t (",
+      #               sym$nu, " = ", round(df, 3),
+      #               (if(!is.null(ncp)) paste(",", sym$sigma, "=", round(ncp,3))
+      #               ), ")\n", sep = "")
     ))
   }
 
@@ -1057,9 +1335,12 @@ vunif <- function(n,
       "u"        = u,
       "x"        = qunif(u, min = min, max = max),
       "quantile" = function(i) qunif(i, min = min, max = max),
-      "text"     = paste("Unif (",
-                     "a = ", round(min, 3), ", ",
-                     "b = ", round(max, 3), ")\n", sep = "")
+      "text"     = as.expression(bquote(
+                        "Unif (" ~ "a" ~ "=" ~ .(round(min, 3)) * ","
+                                 ~ "b" ~ "=" ~ .(round(max, 3)) ~ ")\n"))
+      #"text"     = paste("Unif (",
+      #               "a = ", round(min, 3), ", ",
+      #               "b = ", round(max, 3), ")\n", sep = "")
     ))
   }
 
@@ -1119,9 +1400,12 @@ vweibull <- function(n,
          "u"        = u,
          "x"        = qweibull(u, shape = shape, scale = scale),
          "quantile" = function(i) qweibull(i, shape = shape, scale = scale),
-         "text"     = paste("Weibull (k = ",   round(shape, 3), ", ",
-                        sym$lambda, " = ", round(scale, 3), ")\n",
-                        sep = "")
+         "text"     = as.expression(bquote(
+                            "Weibull (" ~ "k"    ~ "=" ~ .(round(shape, 3)) * ","
+                                        ~ lambad ~ "=" ~ .(round(scale, 3)) ~ ")\n"))
+         #"text"     = paste("Weibull (k = ",   round(shape, 3), ", ",
+         #               sym$lambda, " = ", round(scale, 3), ")\n",
+         #               sep = "")
     ))
   }
 
