@@ -22,6 +22,18 @@
 #' @template i-cont
 #' @template -gamma
 #' @template i-2
+#' @examples
+#'   # overlay visual exploration of ks.test results
+#'   oldpar <- par(no.readonly = TRUE)
+#'   set.seed(54321)
+#'   vals <- igamma(runif(10), 3, 2, showECDF = TRUE, restorePar = FALSE)
+#'   D <- as.numeric(ks.test(vals, "pgamma", 3, 2)$statistic)
+#'   for (x in seq(1.20, 1.60, by = 0.05)) {
+#'     y <- pgamma(x, 3, 2)
+#'     segments(x, y, x, y + D, col = "darkgreen", lwd = 2, xpd = NA)
+#'   }
+#'   par(oldpar) # restore original par values, since restorePar = FALSE above
+#'
 #' @export
 ################################################################################
 igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
@@ -37,18 +49,19 @@ igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
                   sampleColor     = "red3",
                   populationColor = "grey",
                   showTitle       = TRUE,
-                  respectLayout   = FALSE, ...)
+                  respectLayout   = FALSE, 
+                  restorePar      = TRUE,  # add 23 Nov 2023
+                  ...)
 {
   #############################################################################
   
   if(is.null(dev.list()))  dev.new(width=5, height=6)
   
-  warnVal <- options("warn")          # save current warning setting...
-  oldpar  <- par(no.readonly = TRUE)  # save current par settings
+  #warnVal <- options("warn")          # save current warning setting... (del 22 Nov 2023)
 
   #############################################################################
 
-  options(warn = -1)          # suppress warnings
+  #options(warn = -1)          # suppress warnings -- remove RE CRAN req't (del 22 Nov 2023)
 
   if (!is.null(u) && (min(u) <= 0 || max(u) >= 1))  stop("must have 0 < u < 1")
   if (length(u) == 0)  u <- NULL
@@ -62,12 +75,16 @@ igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
 
   checkQuants(minPlotQuantile, maxPlotQuantile, min = 0, maxex = 1)
 
-  options(warn = 1)                   # set to immediate warnings
+  #options(warn = 1)                   # set to immediate warnings (del 22 Nov 2023)
 
   # Check for deprecated parameters
   for (arg in names(list(...))) {
-    if (arg == "maxPlotTime")
-      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+    if (arg == "maxPlotTime") {
+      # mod 23 Nov 2023
+      #warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0",
+              immediate. = TRUE)
+    }
     else stop(paste("Unknown argument '", arg, "'", sep = ""))
   }
 
@@ -78,14 +95,20 @@ igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
   getDistro   <- function(d)  pgamma(d, shape = shape, scale = scale)  #p
   getQuantile <- function(d)  qgamma(d, shape = shape, scale = scale)  #q
 
-  titleStr <- paste("Gamma (",
-    if (missing(rate))
-      paste("k = ",           round(shape, 3), ", ",
-            sym$theta, " = ", round(scale, 3), sep = "")
-    else
-      paste(sym$alpha, " = ", round(shape, 3), ", ",
-            sym$beta,  " = ", round(rate,  3), sep = ""),
-    ")", sep = "")
+  # using plotmath for alpha, beta, theta; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  if (missing(rate)) {
+    titleStr <- as.expression(bquote(bold(
+                    "Gamma (" ~ "k"   ~ "=" ~ .(round(shape, 3)) * ","
+                              ~ theta ~ "=" ~ .(round(scale, 3)) ~ ")"
+                )))
+  } else {
+    # see notation on Wikipedia: https://en.wikipedia.org/wiki/Gamma_distribution
+    titleStr <- as.expression(bquote(bold(
+                    "Gamma (" ~ alpha ~ "=" ~ .(round(shape, 3)) * ","
+                              ~ beta  ~ "=" ~ .(round(scale, 3)) ~ ")"
+                )))
+  }
 
   #############################################################################
 
@@ -104,6 +127,7 @@ igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
     populationColor  = populationColor,
     showTitle        = showTitle,
     respectLayout    = respectLayout,
+    restorePar       = restorePar,    # add 23 Nov 2023
     getDensity       = getDensity,
     getDistro        = getDistro,
     getQuantile      = getQuantile,
@@ -114,11 +138,13 @@ igamma <- function(u = runif(1), shape, rate = 1, scale = 1/rate,
   )
 
   # resetting par and warning settings
-  options(warn = warnVal$warn)
-  if (!all(oldpar$mfrow == par()$mfrow)) {
-    # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
-    par(mfrow = oldpar$mfrow)
-  }
+  #options(warn = warnVal$warn)  # remove RE CRAN req't (del 22 Nov 2023)
+
+  ## using on.exit for par RE CRAN suggest (del 22 Nov 2023)
+  #if (!all(oldpar$mfrow == par()$mfrow)) {
+  #  # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
+  #  par(mfrow = oldpar$mfrow)
+  #}
 
   if (!is.null(out)) return(out)
 }
