@@ -37,18 +37,19 @@ iweibull <- function (u = runif(1), shape, scale = 1,
                 sampleColor     = "red3",
                 populationColor = "grey",
                 showTitle       = TRUE,
-                respectLayout   = FALSE, ...)
+                respectLayout   = FALSE, 
+                restorePar      = TRUE,  # add 23 Nov 2023
+                ...)
 {
   #############################################################################
 
   if(is.null(dev.list()))  dev.new(width=5, height=6)
   
-  warnVal <- options("warn")          # save current warning setting...
-  oldpar  <- par(no.readonly = TRUE)  # save current par settings
+  #warnVal <- options("warn")          # save current warning setting... (del 22 Nov 2023)
 
   #############################################################################
 
-  options(warn = -1)          # suppress warnings
+  #options(warn = -1)          # suppress warnings -- remove RE CRAN req't (del 22 Nov 2023)
 
   if (!is.null(u) && (min(u) <= 0 || max(u) >= 1))  stop("must have 0 < u < 1")
   if (length(u) == 0)  u <- NULL
@@ -58,12 +59,16 @@ iweibull <- function (u = runif(1), shape, scale = 1,
 
   checkQuants(minPlotQuantile, maxPlotQuantile, minex = 0, maxex = 1)
 
-  options(warn = 1)                   # set to immediate warnings
+  #options(warn = 1)                   # set to immediate warnings (del 22 Nov 2023)
 
   # Check for deprecated parameters
   for (arg in names(list(...))) {
-    if (arg == "maxPlotTime")
-      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+    if (arg == "maxPlotTime") {
+      # mod 23 Nov 2023
+      #warning("'maxPlotTime' has been deprecated as of simEd v2.0.0")
+      warning("'maxPlotTime' has been deprecated as of simEd v2.0.0",
+              immediate. = TRUE)
+    }
     else stop(paste("Unknown argument '", arg, "'", sep = ""))
   }
 
@@ -74,9 +79,12 @@ iweibull <- function (u = runif(1), shape, scale = 1,
   getDistro   <- function(d)  pweibull(d, shape, scale)  #p
   getQuantile <- function(d)  qweibull(d, shape, scale)  #q
 
-  titleStr <- paste("Weibull (k = ",   round(shape, 3), ", ",
-                    sym$lambda, " = ", round(scale, 3), ")",
-                    sep = "")
+  # using plotmath for lambda; bquote to use .() to evaluate args;
+  #  in bquote, ~ includes space b/w while * appends w/ no space b/w
+  titleStr <- as.expression(bquote(bold(
+                    "Weibull (" ~ "k"    ~ "=" ~ .(round(shape, 3)) * ","
+                                ~ lambda ~ "=" ~ .(round(scale, 3)) ~ ")"
+              )))
 
   #############################################################################
 
@@ -95,6 +103,7 @@ iweibull <- function (u = runif(1), shape, scale = 1,
     populationColor  = populationColor,
     showTitle        = showTitle,
     respectLayout    = respectLayout,
+    restorePar       = restorePar,    # add 23 Nov 2023
     getDensity       = getDensity,
     getDistro        = getDistro,
     getQuantile      = getQuantile,
@@ -105,11 +114,13 @@ iweibull <- function (u = runif(1), shape, scale = 1,
   )
 
   # resetting par and warning settings
-  options(warn = warnVal$warn)
-  if (!all(oldpar$mfrow == par()$mfrow)) {
-    # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
-    par(mfrow = oldpar$mfrow)
-  }
+  #options(warn = warnVal$warn)  # remove RE CRAN req't (del 22 Nov 2023)
+
+  ## using on.exit for par RE CRAN suggest (del 22 Nov 2023)
+  #if (!all(oldpar$mfrow == par()$mfrow)) {
+  #  # ?par claims "restoring all of [oldpar] is not wise", so reset only mfrow
+  #  par(mfrow = oldpar$mfrow)
+  #}
 
   if (!is.null(out)) return(out)
 }
